@@ -8,7 +8,7 @@ our @ISA = qw( HTTP::Server::PSGI );
 
 use Socket qw(IPPROTO_TCP TCP_NODELAY);
 
-our $VERSION = '0.23_12';
+our $VERSION = '0.23_13';
 
 use Devel::hdb::Logger qw(log);
 
@@ -29,13 +29,58 @@ sub new {
     return $self;
 }
 
+BEGIN {
+if ($ENV{HDB_OVERRIDE_EXIT}) {
+    *CORE::GLOBAL::exit = sub {
+        my $code = shift;
+        print STDERR "In overridden exit: $code\n";
+        CORE::exit($code);
+    };
+}
+}
+
+my $redefined_exit;
+my $redefined__exit;
 sub accept_loop {
     my($self, $app) = @_;
 
+    if (! $redefined__exit and $ENV{HDB_OVERRIDE_EXIT} and defined(&POSIX::_exit)) {
+        print STDERR "Overriding POSIX::_exit...\n";
+        my $old_exit = \&POSIX::_exit;
+        *POSIX::_exit = sub {
+            my $code = shift;
+            print STDERR "In overridden POSIX::_exit: $code\n";
+            $old_exit->($code);
+        };
+        $redefined__exit = 1;
+    }
+    if (! $redefined_exit and $ENV{HDB_OVERRIDE_EXIT} and defined(&POSIX::exit)) {
+        print STDERR "Overriding POSIX::exit...\n";
+        my $old_exit = \&POSIX::exit;
+        *POSIX::exit = sub {
+            my $code = shift;
+            print STDERR "In overridden POSIX::exit: $code\n";
+            $old_exit->($code);
+        };
+        $redefined_exit = 1;
+    }
+
+    my $guard = Devel::hdb::Server::Guard->new(sub { print STDERR "Guard leaving accept_loop()\n" });
     log('entering accept_loop()');
     $app = Plack::Middleware::ContentLength->wrap($app);
     log("got app $app");
 
+    my $signal_catcher = sub {
+        my($sig) = @_;
+        print STDERR "PID $$ caught sig$sig, exiting...\n";
+        CORE::exit(99);
+        print STDERR "$$ Still running!!?!?\n";
+    };
+    my @sigs = qw(HUP INT QUIT ILL TRAP ABRT EMT FPE
+              BUS SEGV SYS PIPE ALRM URG
+              STOP TSTP CONT CHLD IO USR1 USR2 );
+    local @SIG{@sigs} = ($signal_catcher) x @sigs;
+
     ACCEPT_LOOP:
     while (1) {
         log('Top of accept_loop() loop');
@@ -237,4 +282,15 @@ sub _handle_response {
     }
 }
 
+package Devel::hdb::Server::Guard;
+sub new {
+    my($class, $code) = @_;
+    print STDERR "creating guard $code\n";
+    return bless $code, $class;
+}
+sub DESTROY {
+    my $self = shift;
+    $self->();
+}
+
 1;
